# function subroutine to for sub pixel localization using 2D LS fitting#
import numpy as np
from scipy.optimize import curve_fit
import numba


@numba.jit
def twoD_Gaussian(xdata_tuple, amplitude, xo, yo, sigma_x, sigma_y, theta, offset):
    (x, y) = xdata_tuple
    xo = float(xo)
    yo = float(yo)
    a = (np.cos(theta)**2)/(2*sigma_x**2) + (np.sin(theta)**2)/(2*sigma_y**2)
    b = -(np.sin(2*theta))/(4*sigma_x**2) + (np.sin(2*theta))/(4*sigma_y**2)
    c = (np.sin(theta)**2)/(2*sigma_x**2) + (np.cos(theta)**2)/(2*sigma_y**2)
    g = offset + amplitude * \
        np.exp(- (a*((x-xo)**2) + 2*b*(x-xo)*(y-yo) + c*((y-yo)**2)))
    return g.ravel()


def localizer(coordinates, images, window_size, initial_guess, method):

	# Collect the x, y, and intensity values of all the molecules
	molecule_intensity = []
	localized_molecule_x = []
	localized_molecule_y = []
	sigma_x = []
	sigma_y = []

	# Initialize the fitting parameters
	params_fit = [[0]]*5
	
	

	if len(coordinates) > 0:
		
        # Extract x and y coordinates from the input coordinate array
		if method == 'BlobDetection':
			x, y = coordinates[0], coordinates[1]
		else:
			x, y = coordinates

		for j in range(len(x[0])):

			x1, y1 = int(x[0][j]), int(y[0][j])

            # Extract the sub-image
			sub_image = images[y1 - window_size: y1 +
								window_size, x1 - window_size: x1 + window_size]
			sub_image_flat = sub_image.ravel()
			
			# Generate coordinate grids for fitting the Gaussian curves
			xx, yy = np.meshgrid(
				np.arange(sub_image.shape[1]), np.arange(sub_image.shape[0]))
			try:
                # Fit a 2D Gaussian to the y-axis profile
				popt, pcov = curve_fit(
					twoD_Gaussian, (xx, yy), sub_image_flat, p0=initial_guess)

                 # Collect the fitted parameters
				molecule_intensity.append(popt[0])
				localized_molecule_x.append((x1-window_size)+popt[1])
				localized_molecule_y.append((y1-window_size)+popt[2])
				sigma_x.append(popt[3])
				sigma_y.append(popt[4])
                
                # Store the collected fitted parameters
				params_fit = molecule_intensity, localized_molecule_x, localized_molecule_y, sigma_x, sigma_y

			except:
				pass

			

	return params_fit
